import type { Root } from "@arktype/schema";
import { type ErrorMessage, type join, type nominal } from "@arktype/util";
import type { ParseContext } from "../scope.js";
import { writeUnclosedGroupMessage } from "./string/reduce/shared.js";
import type { StaticState, state } from "./string/reduce/static.js";
import { writeUnexpectedCharacterMessage } from "./string/shift/operator/operator.js";
import { Scanner } from "./string/shift/scanner.js";
import { parseUntilFinalizer } from "./string/string.js";
export type GenericDeclaration<name extends string = string, params extends string = string> = `${name}<${params}>`;
export type GenericParamsParseError<message extends string = string> = [
    nominal<message, "InvalidGenericParameters">
];
export declare const parseGenericParams: (def: string) => string[];
export type parseGenericParams<def extends string> = parseParamsRecurse<def, "", [
]> extends infer result extends string[] ? "" extends result[number] ? GenericParamsParseError<emptyGenericParameterMessage> : result : never;
export declare const emptyGenericParameterMessage = "An empty string is not a valid generic parameter name";
export type emptyGenericParameterMessage = typeof emptyGenericParameterMessage;
type parseParamsRecurse<unscanned extends string, param extends string, result extends string[]> = unscanned extends `${infer lookahead}${infer nextUnscanned}` ? lookahead extends "," ? parseParamsRecurse<nextUnscanned, "", [...result, param]> : lookahead extends Scanner.WhiteSpaceToken ? param extends "" ? parseParamsRecurse<Scanner.skipWhitespace<nextUnscanned>, "", result> : Scanner.skipWhitespace<nextUnscanned> extends `${infer nextNonWhitespace}${infer rest}` ? nextNonWhitespace extends "," ? parseParamsRecurse<rest, "", [...result, param]> : GenericParamsParseError<writeUnexpectedCharacterMessage<nextNonWhitespace, ",">> : [
    ...result,
    param
] : parseParamsRecurse<nextUnscanned, `${param}${lookahead}`, result> : param extends "" ? result : [...result, param];
export type ParsedArgs<result extends unknown[] = unknown[], unscanned extends string = string> = {
    result: result;
    unscanned: unscanned;
};
export declare const parseGenericArgs: (name: string, params: string[], unscanned: string, ctx: ParseContext) => ParsedArgs<Root[], string>;
export type parseGenericArgs<name extends string, params extends string[], unscanned extends string, $, args> = parseGenericArgsRecurse<name, params, unscanned, $, args, [], []>;
declare const parseGenericArgsRecurse: (name: string, params: string[], unscanned: string, ctx: ParseContext, argDefs: string[], argNodes: Root[]) => ParsedArgs<Root[]>;
type parseGenericArgsRecurse<name extends string, params extends string[], unscanned extends string, $, args, argDefs extends string[], argAsts extends unknown[]> = parseUntilFinalizer<state.initialize<unscanned>, $, args> extends infer finalArgState extends StaticState ? {
    defs: [
        ...argDefs,
        finalArgState["scanned"] extends `${infer def}${"," | ">"}` ? def : finalArgState["scanned"]
    ];
    asts: [...argAsts, finalArgState["root"]];
    unscanned: finalArgState["unscanned"];
} extends {
    defs: infer nextDefs extends string[];
    asts: infer nextAsts extends unknown[];
    unscanned: infer nextUnscanned extends string;
} ? finalArgState["finalizer"] extends ">" ? nextAsts["length"] extends params["length"] ? ParsedArgs<nextAsts, nextUnscanned> : state.error<writeInvalidGenericArgsMessage<name, params, nextDefs>> : finalArgState["finalizer"] extends "," ? parseGenericArgsRecurse<name, params, nextUnscanned, $, args, nextDefs, nextAsts> : finalArgState["finalizer"] extends ErrorMessage ? finalArgState : state.error<writeUnclosedGroupMessage<">">> : never : never;
export declare const writeInvalidGenericArgsMessage: <name extends string, params extends string[], argDefs extends string[]>(name: name, params: params, argDefs: argDefs) => string;
export type writeInvalidGenericArgsMessage<name extends string, params extends string[], argDefs extends string[]> = `${name}<${join<params, ", ">}> requires exactly ${params["length"]} args (got ${argDefs["length"]}${argDefs["length"] extends 0 ? "" : `: ${join<argDefs, ",">}`})`;
export {};
//# sourceMappingURL=generic.d.ts.map