import { throwParseError } from "@arktype/util";
import { DynamicState } from "./string/reduce/dynamic.js";
import { writeUnclosedGroupMessage } from "./string/reduce/shared.js";
import { writeUnexpectedCharacterMessage } from "./string/shift/operator/operator.js";
import { Scanner } from "./string/shift/scanner.js";
import { parseUntilFinalizer } from "./string/string.js";
export const parseGenericParams = (def) => parseGenericParamsRecurse(new Scanner(def));
export const emptyGenericParameterMessage = `An empty string is not a valid generic parameter name`;
const parseGenericParamsRecurse = (scanner) => {
    const param = scanner.shiftUntilNextTerminator();
    if (param === "") {
        throwParseError(emptyGenericParameterMessage);
    }
    scanner.shiftUntilNonWhitespace();
    const nextNonWhitespace = scanner.shift();
    return nextNonWhitespace === ""
        ? [param]
        : nextNonWhitespace === ","
            ? [param, ...parseGenericParamsRecurse(scanner)]
            : throwParseError(writeUnexpectedCharacterMessage(nextNonWhitespace, ","));
};
export const parseGenericArgs = (name, params, unscanned, ctx) => parseGenericArgsRecurse(name, params, unscanned, ctx, [], []);
const parseGenericArgsRecurse = (name, params, unscanned, ctx, argDefs, argNodes) => {
    const s = parseUntilFinalizer(new DynamicState(unscanned, ctx));
    // remove the finalizing token from the argDef
    argDefs.push(s.scanner.scanned.slice(0, -1));
    argNodes.push(s.root);
    const nextUnscanned = s.scanner.unscanned;
    if (s.finalizer === ">") {
        if (argNodes.length === params.length) {
            return {
                result: argNodes,
                unscanned: nextUnscanned
            };
        }
        else {
            return s.error(writeInvalidGenericArgsMessage(name, params, argDefs));
        }
    }
    else if (s.finalizer === ",") {
        return parseGenericArgsRecurse(name, params, nextUnscanned, ctx, argDefs, argNodes);
    }
    return s.error(writeUnclosedGroupMessage(">"));
};
export const writeInvalidGenericArgsMessage = (name, params, argDefs) => `${name}<${params.join(", ")}> requires exactly ${params.length} args (got ${argDefs.length}${argDefs.length === 0 ? "" : ": " + argDefs.join(", ")})`;
//# sourceMappingURL=generic.js.map