import { node } from "@arktype/schema";
import { stringify } from "@arktype/util";
import { parseEntry } from "./shared.js";
const stringAndSymbolicEntriesOf = (o) => [
    ...Object.entries(o),
    ...Object.getOwnPropertySymbols(o).map((k) => [k, o[k]])
];
export const parseObjectLiteral = (def, ctx) => {
    const required = [];
    const optional = [];
    for (const entry of stringAndSymbolicEntriesOf(def)) {
        const result = parseEntry(entry);
        ctx.path.push(`${typeof result.innerKey === "symbol"
            ? `[${stringify(result.innerKey)}]`
            : result.innerKey}`);
        const valueNode = ctx.scope.parse(result.innerValue, ctx);
        if (result.kind === "optional") {
            optional.push({
                key: result.innerKey,
                value: valueNode
            });
        }
        else {
            required.push({
                key: result.innerKey,
                value: valueNode
            });
        }
        ctx.path.pop();
    }
    return node({
        basis: "object",
        required,
        optional
    });
};
export const writeInvalidPropertyKeyMessage = (indexDef) => `Indexed key definition '${indexDef}' must be a string, number or symbol`;
//# sourceMappingURL=objectLiteral.js.map