import type { Root } from "@arktype/schema";
import { type ErrorMessage } from "@arktype/util";
import type { ParseContext } from "../../scope.js";
import type { inferAst } from "../semantic/semantic.js";
import { DynamicState, type DynamicStateWithRoot } from "./reduce/dynamic.js";
import type { StringifiablePrefixOperator } from "./reduce/shared.js";
import type { StaticState, state } from "./reduce/static.js";
import { parseOperand } from "./shift/operand/operand.js";
import { parseOperator, writeUnexpectedCharacterMessage } from "./shift/operator/operator.js";
export declare const parseString: (def: string, ctx: ParseContext) => Root;
/**
 * Try to parse the definition from right to left using the most common syntax.
 * This can be much more efficient for simple definitions.
 */
export type parseString<def extends string, $, args> = def extends keyof $ ? def : def extends `${infer child}[]` ? child extends keyof $ ? [child, "[]"] : fullStringParse<def, $, args> : fullStringParse<def, $, args>;
export type inferString<def extends string, $, args> = inferAst<parseString<def, $, args>, $, args>;
export type BaseCompletions<$, args, otherSuggestions extends string = never> = (keyof $ & string) | (keyof args & string) | StringifiablePrefixOperator | otherSuggestions;
export declare const fullStringParse: (def: string, ctx: ParseContext) => import("@arktype/schema").RootNode<"domain", unknown> | import("@arktype/schema").RootNode<"proto", unknown> | import("@arktype/schema").RootNode<"unit", unknown> | import("@arktype/schema").RootNode<"morph", unknown> | import("@arktype/schema").RootNode<"intersection", unknown> | import("@arktype/schema").RootNode<"union", unknown>;
type fullStringParse<def extends string, $, args> = extractFinalizedResult<parseUntilFinalizer<state.initialize<def>, $, args>>;
export declare const parseUntilFinalizer: (s: DynamicState) => DynamicStateWithRoot;
export type parseUntilFinalizer<s extends StaticState, $, args> = s["finalizer"] extends undefined ? parseUntilFinalizer<next<s, $, args>, $, args> : s;
declare const next: (s: DynamicState) => void;
type next<s extends StaticState, $, args> = s["root"] extends undefined ? parseOperand<s, $, args> : parseOperator<s, $, args>;
export type extractFinalizedResult<s extends StaticState> = s["finalizer"] extends ErrorMessage ? s["finalizer"] : s["finalizer"] extends "" ? s["root"] : state.error<writeUnexpectedCharacterMessage<`${s["finalizer"]}`>>;
export {};
//# sourceMappingURL=string.d.ts.map