import { throwParseError } from "@arktype/util";
import { writeUnsatisfiableExpressionError } from "../semantic/validate.js";
import { DynamicState } from "./reduce/dynamic.js";
import { parseOperand } from "./shift/operand/operand.js";
import { parseOperator, writeUnexpectedCharacterMessage } from "./shift/operator/operator.js";
export const parseString = (def, ctx) => ctx.scope.maybeResolveNode(def) ??
    ((def.endsWith("[]") &&
        ctx.scope.maybeResolveNode(def.slice(0, -2))?.array()) ||
        fullStringParse(def, ctx));
export const fullStringParse = (def, ctx) => {
    const s = new DynamicState(def, ctx);
    parseOperand(s);
    const result = parseUntilFinalizer(s).root;
    s.scanner.shiftUntilNonWhitespace();
    if (s.scanner.lookahead) {
        // throw a parse error if non-whitespace characters made it here without being parsed
        throwParseError(writeUnexpectedCharacterMessage(s.scanner.lookahead));
    }
    // TODO: would this ever happen?
    return result.isNever()
        ? throwParseError(writeUnsatisfiableExpressionError(def))
        : result;
};
export const parseUntilFinalizer = (s) => {
    while (s.finalizer === undefined) {
        next(s);
    }
    return s;
};
const next = (s) => s.hasRoot() ? parseOperator(s) : parseOperand(s);
//# sourceMappingURL=string.js.map