import type { inferred } from "@arktype/schema";
import type { MatchParser, WhenParser } from "../match.js";
import { Scope, type Module, type ScopeParser } from "../scope.js";
import type { DeclarationParser, DefinitionParser, TypeParser } from "../type.js";
import { type InferredJsObjects } from "./jsObjects.js";
import { type ParsingModule } from "./parsing.js";
import { type InferredTsGenerics } from "./tsGenerics.js";
import { type InferredTsKeywords } from "./tsKeywords.js";
import { type InferredValidation } from "./validation.js";
/** Root scopes can be inferred automatically from node definitions, but
 * explicitly typing them can improve responsiveness */
export type RootScope<exports> = Scope<{
    exports: exports;
    locals: {};
    ambient: {};
}>;
export type ArkResolutions = {
    exports: Ark;
    locals: {};
    ambient: Ark;
};
export declare const ark: Scope<ArkResolutions>;
export declare const arktypes: Module<ArkResolutions>;
export interface Ark extends InferredTsKeywords, InferredJsObjects, InferredValidation, InferredTsGenerics {
    parse: ParsingModule;
}
export declare const scope: ScopeParser<{}, Ark>;
export declare const type: TypeParser<Ark>;
export declare const match: MatchParser<Ark>;
export declare const when: WhenParser<Ark>;
export declare namespace type {
    type cast<to> = {
        [inferred]?: to;
    };
}
export declare const define: DefinitionParser<Ark>;
export declare const declare: DeclarationParser<Ark>;
//# sourceMappingURL=ark.d.ts.map