import { node } from "@arktype/schema";
import { wellFormedIntegerMatcher, wellFormedNumberMatcher } from "@arktype/util";
import { Scope } from "../scope.js";
import { parsedDate } from "./utils/date.js";
const number = node({
    in: {
        basis: "string",
        pattern: wellFormedNumberMatcher,
        description: "a well-formed numeric string"
    },
    morph: (s) => parseFloat(s)
});
const integer = node({
    in: {
        basis: "string",
        pattern: wellFormedIntegerMatcher
    },
    morph: (s) => {
        // if (!isWellFormedInteger(s)) {
        // 	return problems.mustBe("a well-formed integer string")
        // }
        // const parsed = parseInt(s)
        // return Number.isSafeInteger(parsed)
        // 	? parsed
        // 	: problems.mustBe(
        // 			"an integer in the range Number.MIN_SAFE_INTEGER to Number.MAX_SAFE_INTEGER"
        // 	  )
        return parseInt(s);
    }
});
const url = node({
    in: {
        basis: "string",
        description: "a valid URL"
    },
    morph: (s) => {
        return new URL(s);
        // try {
        // 	return new URL(s)
        // } catch {
        // 	return state.mustBe("a valid URL", s, state.basePath)
        // }
    }
});
const json = node({
    in: {
        basis: "string",
        description: "a JSON-parsable string"
    },
    morph: (s) => JSON.parse(s)
});
const date = parsedDate;
export const parsing = Scope.root({
    url,
    number,
    integer,
    date,
    json
});
export const parsingModule = parsing.export();
//# sourceMappingURL=parsing.js.map