import { arkKind, inferred, type BaseAttributes, type CheckResult, type Morph, type Out, type Predicate, type Root, type UnknownNode, type inferMorphOut, type inferNarrow } from "@arktype/schema";
import { CompiledFunction, type BuiltinObjectKind, type BuiltinObjects, type Constructor, type Json, type Primitive, type conform } from "@arktype/util";
import type { inferDefinition, validateDeclared, validateDefinition } from "./parser/definition.js";
import { parseGenericParams, type GenericParamsParseError } from "./parser/generic.js";
import type { inferIntersection } from "./parser/semantic/intersections.js";
import type { IndexOneOperator, IndexZeroOperator, MorphAst, TupleInfixOperator } from "./parser/tuple.js";
import { bindThis, type Module, type Scope } from "./scope.js";
export type TypeParser<$> = {
    <const def>(def: validateTypeRoot<def, $>): Type<inferTypeRoot<def, $>, $>;
    <const zero, const one, const rest extends readonly unknown[]>(_0: zero extends IndexZeroOperator ? zero : validateTypeRoot<zero, $>, _1: zero extends "keyof" ? validateTypeRoot<one, $> : zero extends "instanceof" ? conform<one, Constructor> : zero extends "===" ? conform<one, unknown> : conform<one, IndexOneOperator>, ..._2: zero extends "===" ? rest : zero extends "instanceof" ? conform<rest, readonly Constructor[]> : one extends TupleInfixOperator ? one extends ":" ? [Predicate<extractIn<inferTypeRoot<zero, $>>>] : one extends "=>" ? [
        Morph<extractOut<inferTypeRoot<zero, $>>, unknown>
    ] : one extends "@" ? [string | BaseAttributes] : [validateTypeRoot<rest[0], $>] : []): Type<inferTypeRoot<[zero, one, ...rest], $>, $>;
    <params extends string, const def>(params: `<${validateParameterString<params>}>`, def: validateDefinition<def, $, {
        [param in parseGenericParams<params>[number]]: unknown;
    }>): Generic<parseGenericParams<params>, def, $>;
};
export type DeclarationParser<$> = <preinferred>() => {
    type: <def>(def: validateDeclared<preinferred, def, $, bindThis<def>>) => Type<preinferred, $>;
};
export declare const createTypeParser: <$>(scope: Scope) => TypeParser<$>;
export type ArkKinds = {
    node: UnknownNode;
    generic: Generic;
    module: Module;
};
export declare const addArkKind: <kind extends keyof ArkKinds>(value: Omit<ArkKinds[kind], typeof arkKind> & {
    [arkKind]?: kind;
}, kind: kind) => ArkKinds[kind];
export type arkKind = typeof arkKind;
export type ArkKind = keyof ArkKinds;
export declare const hasArkKind: <kind extends keyof ArkKinds>(value: unknown, kind: kind) => value is ArkKinds[kind];
export type DefinitionParser<$> = <def>(def: validateDefinition<def, $, bindThis<def>>) => def;
export type KeyCheckKind = "distilled" | "strict" | "loose";
export type TypeConfig = {
    keys?: KeyCheckKind;
    mustBe?: string;
};
export declare class Type<t = unknown, $ = any> extends CompiledFunction<(data: unknown) => CheckResult<extractOut<t>>> {
    definition: unknown;
    scope: Scope;
    [inferred]: t;
    inferMorph: t;
    infer: extractOut<t>;
    inferIn: extractIn<t>;
    config: TypeConfig;
    root: Root<t>;
    condition: string;
    alias: string;
    allows: this["root"]["allows"];
    json: Json;
    constructor(definition: unknown, scope: Scope);
    configure(config: TypeConfig): this;
    from(literal: this["infer"]): this["infer"];
    fromIn(literal: this["inferIn"]): this["inferIn"];
    and<def>(def: validateTypeRoot<def, $>): Type<inferIntersection<t, inferTypeRoot<def, $>>, $>;
    or<def>(def: validateTypeRoot<def, $>): Type<t | inferTypeRoot<def, $>, $>;
    morph<morph extends Morph<extractOut<t>>>(morph: morph): Type<(In: this["inferIn"]) => Out<inferMorphOut<ReturnType<morph>>>, $>;
    morph<morph extends Morph<extractOut<t>>, def>(morph: morph, outValidator: validateTypeRoot<def, $>): Type<(In: this["inferIn"]) => Out<extractOut<inferTypeRoot<def, $>>>, $>;
    narrow<def extends Predicate<extractOut<t>>>(def: def): Type<includesMorphs<t> extends true ? (In: this["inferIn"]) => Out<inferNarrow<this["infer"], def>> : inferNarrow<this["infer"], def>, $>;
    array(): Type<t[], $>;
    keyof(): Type<keyof this["inferIn"], $>;
    assert(data: unknown): extractOut<t>;
    equals<other>(other: Type<other>): this is Type<other, $>;
    extends<other>(other: Type<other>): this is Type<other, $>;
}
export type validateTypeRoot<def, $> = validateDefinition<def, $, bindThis<def>>;
export type inferTypeRoot<def, $> = inferDefinition<def, $, bindThis<def>>;
type validateParameterString<params extends string> = parseGenericParams<params> extends GenericParamsParseError<infer message> ? message : params;
export declare const validateUninstantiatedGeneric: (g: Generic) => Generic<string[], unknown, any>;
export declare const generic: (parameters: string[], definition: unknown, scope: Scope) => Generic<string[], unknown, any>;
export type GenericProps<params extends string[] = string[], def = unknown, $ = any> = {
    [arkKind]: "generic";
    $: $;
    parameters: params;
    definition: def;
    scope: Scope;
};
export type BoundArgs = Record<string, Root>;
export type Generic<params extends string[] = string[], def = unknown, $ = any> = {
    <args>(...args: conform<args, {
        [i in keyof params]: validateTypeRoot<args[i & keyof args], $>;
    }>): Type<inferDefinition<def, $, bindGenericInstantiation<params, $, args>>, $>;
} & GenericProps<params, def, $>;
type bindGenericInstantiation<params extends string[], $, args> = {
    [i in keyof params & `${number}` as params[i]]: inferTypeRoot<args[i & keyof args], $>;
};
export type extractIn<t> = includesMorphs<t> extends true ? extractMorphs<t, "in"> : t;
export type extractOut<t> = includesMorphs<t> extends true ? extractMorphs<t, "out"> : t;
type includesMorphs<t> = [
    t,
    extractMorphs<t, "in">,
    t,
    extractMorphs<t, "out">
] extends [extractMorphs<t, "in">, t, extractMorphs<t, "out">, t] ? false : true;
type extractMorphs<t, io extends "in" | "out"> = t extends MorphAst<infer i, infer o> ? io extends "in" ? i : o : t extends TerminallyInferredObjectKind | Primitive ? t : {
    [k in keyof t]: extractMorphs<t[k], io>;
};
/** Objects we don't want to expand during inference like Date or Promise */
type TerminallyInferredObjectKind = ReturnType<ArkConfig["preserve"]> | BuiltinObjects[Exclude<BuiltinObjectKind, "Object" | "Array">];
export {};
//# sourceMappingURL=type.d.ts.map